import { IParam } from "@core";
import { ListControlState } from "./list-control-state";
import { MDControl } from "../md-control";

/**
 * @description 列表部件
 * @export
 * @class ListControl
 * @extends {MDControl}
 */
export class ListControl extends MDControl {

    /**
     * @description 部件状态
     * @type {ListControlState}
     * @memberof ListControl
     */
    public declare state: ListControlState;

    /**
     * @description 触发界面行为
     * @protected
     * @param {IParam} item 列表数据
     * @param {IParam} action 界面行为
     * @param {MouseEvent} event 鼠标源事件
     * @memberof ListControl
     */
    protected onUIAction(item: IParam, action: IParam, event: MouseEvent) {
        if (!item || !action) {
            console.warn("行为执行参数不足");
            return;
        }
        const inputParam = {
            context: this.state.context,
            viewParams: this.state.viewParams,
            data: [item],
            event: event,
            actionEnvironment: this
        };
        // 执行行为
        App.getAppActionService().execute(action, inputParam);
    }

    /**
     * @description 使用自定义模块
     * @protected
     * @return {*} 
     * @memberof ListControl
     */
    protected useCustom() {
        /**
         * 列表项选中
         * @param item 
         * @param event 
         */
        const onListItemSelected = (item: IParam, event: MouseEvent) => {
            const { isMultiple } = this.state;
            const { selectedData } = toRefs(this.state);
            const index = selectedData.value.findIndex((selection: any) => selection.srfkey === item.srfkey);
            //  存在选中则删除
            if (index !== -1) {
                selectedData.value.splice(index, 1);
            } else {
                //  单选清空已选中
                if (!isMultiple) {
                    selectedData.value.splice(index, selectedData.value.length);
                }
                selectedData.value.push(item);
            }
            this.emit('ctrlEvent', { tag: this.props.name, action: 'selectionchange', data: selectedData });
        }

        /**
         * 是否选中
         * @param item 列表项
         * @returns 
         */
        const isSelected = (item: IParam): boolean => {
            const { selectedData } = toRefs(this.state);
            return selectedData.value.findIndex((selection: any) => selection.srfkey === item.srfkey) !== -1;
        }

        /**
         * 加载更多
         * @param event 
         */
        const loadMore = (event: MouseEvent) => {
            const { mdCtrlPaging } = this.state;
            let { current, pageSize } = mdCtrlPaging;
            current += 1;
            this.load();
        }
        return {
            onListItemSelected: onListItemSelected.bind(this),
            isSelected: isSelected.bind(this),
            loadMore: loadMore.bind(this)
        }
    }

    /**
     * @description 安装部件所有功能模块的方法
     * @return {*} 
     * @memberof ListControl
     */
    public moduleInstall() {
        const superParams = super.moduleInstall();
        return {
            ...superParams,
            onUIAction: this.onUIAction.bind(this),
            useCustom: this.useCustom()
        }
    }

}