import { IActionParam, IParam } from "@core";
import { MainControl } from "../main-control";
import { PanelControlState } from "./panel-control-state";

/**
 * @description 面板部件
 * @export
 * @class PanelControl
 * @extends {MainControl}
 */
export class PanelControl extends MainControl {

    /**
     * @description 部件状态
     * @type {FormControlState}
     * @memberof PanelControl
     */
    public declare state: PanelControlState;

    /**
     * @description 处理导航数据模块
     * @protected
     * @memberof PanelControl
     */
    protected useNavDatas() {
        const navDatas: any = toRef(this.props, 'navDatas');
        if (this.props.panelType === 'LAYOUT') {
            this.load();
        }
        watch(navDatas, (newVal: any, oldVal: any) => {
            this.load();
        });
        this.state.navDatas = navDatas;
    }

    /**
     * @description 数据加载
     * @protected
     * @param {IParam} [opts={}]
     * @memberof PanelControl
     */
    protected async load(opts: IParam = {}) { }

    /**
     * @description 使用数据加载模块
     * @memberof PanelControl
     */
    public useLoad() {
        const { viewSubject, controlName } = this.state;
        const load = async (opts: IParam = {}) => {
            const { dataMode } = this.state;
            const { navDatas, data } = toRefs(this.state);
            if (dataMode === 0) {
                //  不获取，使用传入数据
                if (navDatas && navDatas.value && navDatas.value.length) {
                    data.value = navDatas.value[0];
                }
            } else if (dataMode === 1) {
                //  存在传入数据时，不获取
                if (navDatas && navDatas.value && navDatas.value.length) {
                    data.value = navDatas.value[0];
                } else {
                    await this.loadPanelData(opts);
                }
            } else if (dataMode === 2) {
                //  始终获取
                await this.loadPanelData(opts);
            }
        }
        this.load = load;

        // 订阅viewSubject,监听load行为
        if (viewSubject) {
            let subscription = viewSubject.subscribe(({ tag, action, data }: IActionParam) => {
                if (Object.is(controlName, tag) && Object.is('load', action)) {
                    load(data);
                }
            });

            // 部件卸载时退订viewSubject
            onUnmounted(() => {
                subscription.unsubscribe();
            });
        }
        return load;
    }

    private loadPanelData(opts: IParam = {}) {

    }

    /**
     * @description 安装部件所有功能模块的方法
     * @return {*} 
     * @memberof PanelControl
     */
    public moduleInstall() {
        const superParams = super.moduleInstall();
        this.useNavDatas();
        return {
            ...superParams,
            load: this.useLoad(),
        }
    }

}